@@ -1,8 +1,9 @@
 from checkov.common.models.enums import CheckResult, CheckCategories
-from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck
+from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck
+from checkov.common.models.consts import ANY_VALUE
 
 
-class SQSQueueEncryption(BaseResourceCheck):
+class SQSQueueEncryption(BaseResourceValueCheck):
     def __init__(self):
         name = "Ensure all data stored in the SQS queue is encrypted"
         id = "CKV_AWS_27"
@@ -10,11 +11,11 @@ class SQSQueueEncryption(BaseResourceCheck):
         categories = [CheckCategories.ENCRYPTION]
         super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)
 
-    def scan_resource_conf(self, conf):
-        if conf.get('Properties'):
-            if conf['Properties'].get('KmsMasterKeyId'):
-                return CheckResult.PASSED
-        return CheckResult.FAILED
+    def get_inspected_key(self):
+        return 'Properties/KmsMasterKeyId'
+
+    def get_expected_value(self):
+        return ANY_VALUE
 
 
 check = SQSQueueEncryption()
