@@ -16,7 +16,7 @@ K8_POSSIBLE_ENDINGS = [".yaml", ".yml", ".json"]
 class Runner(BaseRunner):
     check_type = "kubernetes"
 
-    def run(self, root_folder, external_checks_dir=None, files=None, runner_filter=RunnerFilter(), collect_skip_comments=True):
+    def run(self, root_folder, external_checks_dir=None, files=None, runner_filter=RunnerFilter(), collect_skip_comments=True, helmChart=None):
         report = Report(self.check_type)
         definitions = {}
         definitions_raw = {}
@@ -24,7 +24,7 @@ class Runner(BaseRunner):
         files_list = []
         if external_checks_dir:
             for directory in external_checks_dir:
-                registry.load_external_checks(directory, runner_filter)
+                registry.load_external_checks(directory)
 
         if files:
             for file in files:
