@@ -1,5 +1,7 @@
 import logging
 import os
+
+from checkov.cloudformation import cfn_utils
 from checkov.cloudformation.context_parser import ContextParser as CfnContextParser
 from checkov.serverless.base_registry import EntityDetails
 from checkov.serverless.parsers.context_parser import ContextParser as SlsContextParser
@@ -46,7 +48,7 @@ class Runner(BaseRunner):
         files_list = []
         if external_checks_dir:
             for directory in external_checks_dir:
-                function_registry.load_external_checks(directory, runner_filter)
+                function_registry.load_external_checks(directory)
 
         if files:
             for file in files:
@@ -81,6 +83,18 @@ class Runner(BaseRunner):
         definitions_raw = {k: v for k, v in definitions_raw.items() if k in definitions.keys()}
 
         for sls_file, sls_file_data in definitions.items():
+
+            # There are a few cases here. If -f was used, there could be a leading / because it's an absolute path,
+            # or there will be no leading slash; root_folder will always be none.
+            # If -d is used, root_folder will be the value given, and -f will start with a / (hardcoded above).
+            # The goal here is simply to get a valid path to the file (which sls_file does not always give).
+            if sls_file[0] == '/':
+                path_to_convert = (root_folder + sls_file) if root_folder else sls_file
+            else:
+                path_to_convert = (os.path.join(root_folder, sls_file)) if root_folder else sls_file
+
+            file_abs_path = os.path.abspath(path_to_convert)
+
             if not isinstance(sls_file_data, dict_node):
                 continue
 
@@ -93,6 +107,9 @@ class Runner(BaseRunner):
                     if not isinstance(resource, dict_node):
                         continue
                     cf_resource_id = cf_context_parser.extract_cf_resource_id(resource, resource_name)
+                    if not cf_resource_id:
+                        # Not Type attribute for resource
+                        continue
                     entity_lines_range, entity_code_lines = cf_context_parser.extract_cf_resource_code_lines(
                         resource)
                     if entity_lines_range and entity_code_lines:
@@ -100,14 +117,15 @@ class Runner(BaseRunner):
                         # TODO - Variable Eval Message!
                         variable_evaluations = {}
 
-                        results = cfn_registry.scan(sls_file, {resource_name: resource}, skipped_checks,
-                                                    runner_filter)
+                        entity = {resource_name: resource}
+                        results = cfn_registry.scan(sls_file, entity, skipped_checks, runner_filter)
+                        tags = cfn_utils.get_resource_tags(entity, cfn_registry)
                         for check, check_result in results.items():
                             record = Record(check_id=check.id, check_name=check.name, check_result=check_result,
                                             code_block=entity_code_lines, file_path=sls_file,
                                             file_line_range=entity_lines_range,
                                             resource=cf_resource_id, evaluations=variable_evaluations,
-                                            check_class=check.__class__.__module__)
+                                            check_class=check.__class__.__module__, file_abs_path=file_abs_path, entity_tags=tags)
                             report.add_record(record=record)
 
             sls_context_parser = SlsContextParser(sls_file, sls_file_data, definitions_raw[sls_file])
@@ -129,15 +147,15 @@ class Runner(BaseRunner):
                             # function data from the provider block since logically that's what serverless
                             # does. This allows checks to see what the complete data would be.
                             sls_context_parser.enrich_function_with_provider(item_name)
-                        results = registry.scan(sls_file,
-                                                EntityDetails(sls_context_parser.provider_type, item_content),
-                                                skipped_checks, runner_filter)
+                        entity = EntityDetails(sls_context_parser.provider_type, item_content)
+                        results = registry.scan(sls_file, entity, skipped_checks, runner_filter)
+                        tags = cfn_utils.get_resource_tags(entity, registry)
                         for check, check_result in results.items():
                             record = Record(check_id=check.id, check_name=check.name, check_result=check_result,
                                             code_block=entity_code_lines, file_path=sls_file,
                                             file_line_range=entity_lines_range,
                                             resource=item_name, evaluations=variable_evaluations,
-                                            check_class=check.__class__.__module__)
+                                            check_class=check.__class__.__module__, file_abs_path=file_abs_path, entity_tags=tags)
                             report.add_record(record=record)
             # Sub-sections that are a single item
             for token, registry in SINGLE_ITEM_SECTIONS:
@@ -150,15 +168,15 @@ class Runner(BaseRunner):
 
                 skipped_checks = CfnContextParser.collect_skip_comments(entity_code_lines)
                 variable_evaluations = {}
-                results = registry.scan(sls_file,
-                                        EntityDetails(sls_context_parser.provider_type, item_content),
-                                        skipped_checks, runner_filter)
+                entity = EntityDetails(sls_context_parser.provider_type, item_content)
+                results = registry.scan(sls_file, entity, skipped_checks, runner_filter)
+                tags = cfn_utils.get_resource_tags(entity, registry)
                 for check, check_result in results.items():
                     record = Record(check_id=check.id, check_name=check.name, check_result=check_result,
                                     code_block=entity_code_lines, file_path=sls_file,
                                     file_line_range=entity_lines_range,
                                     resource=token, evaluations=variable_evaluations,
-                                    check_class=check.__class__.__module__)
+                                    check_class=check.__class__.__module__, file_abs_path=file_abs_path, entity_tags=tags)
                     report.add_record(record=record)
 
             # "Complete" checks
@@ -167,9 +185,9 @@ class Runner(BaseRunner):
             if entity_lines_range:
                 skipped_checks = CfnContextParser.collect_skip_comments(entity_code_lines)
                 variable_evaluations = {}
-                results = complete_registry.scan(sls_file,
-                                                 EntityDetails(sls_context_parser.provider_type, sls_file_data),
-                                                 skipped_checks, runner_filter)
+                entity = EntityDetails(sls_context_parser.provider_type, sls_file_data)
+                results = complete_registry.scan(sls_file, entity, skipped_checks, runner_filter)
+                tags = cfn_utils.get_resource_tags(entity, complete_registry)
                 for check, check_result in results.items():
                     record = Record(check_id=check.id, check_name=check.name, check_result=check_result,
                                     code_block=[],              # Don't show, could be large
@@ -177,7 +195,7 @@ class Runner(BaseRunner):
                                     file_line_range=entity_lines_range,
                                     resource="complete",        # Weird, not sure what to put where
                                     evaluations=variable_evaluations,
-                                    check_class=check.__class__.__module__)
+                                    check_class=check.__class__.__module__, file_abs_path=file_abs_path, entity_tags=tags)
                     report.add_record(record=record)
 
         return report
