@@ -12,9 +12,6 @@ from checkov.terraform.context_parsers.registry import parser_registry
 # Allow the evaluation of empty variables
 from checkov.terraform.plan_parser import parse_tf_plan
 
-LOG_LEVEL = os.getenv('LOG_LEVEL', 'WARNING').upper()
-logging.basicConfig(level=LOG_LEVEL)
-
 
 class Runner(BaseRunner):
     check_type = "terraform_plan"
@@ -35,7 +32,7 @@ class Runner(BaseRunner):
         parsing_errors = {}
         if external_checks_dir:
             for directory in external_checks_dir:
-                resource_registry.load_external_checks(directory, runner_filter)
+                resource_registry.load_external_checks(directory)
 
         if root_folder:
             files = [] if not files else files
@@ -63,6 +60,8 @@ class Runner(BaseRunner):
                     self.tf_definitions = tf_definitions
                     self.template_lines = template_lines
                     self.check_tf_definition(report, runner_filter)
+                else:
+                    logging.debug(f'Failed to load {file} as is not a .json file, skipping')
 
         report.add_parsing_errors(parsing_errors.keys())
 
