@@ -1,10 +1,14 @@
+import copy
 import dataclasses
 import logging
 import os
-from typing import Dict, Optional, Tuple
+from typing import Dict, Optional, Tuple, List
 
 import dpath.util
 
+from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
+from checkov.common.models.enums import CheckResult
+from checkov.common.output.graph_record import GraphRecord
 from checkov.common.output.record import Record
 from checkov.common.output.report import Report
 from checkov.common.util import dict_utils
@@ -15,26 +19,39 @@ from checkov.terraform.checks.data.registry import data_registry
 from checkov.terraform.checks.module.registry import module_registry
 from checkov.terraform.checks.provider.registry import provider_registry
 from checkov.terraform.checks.resource.registry import resource_registry
+from checkov.terraform.checks_infra.checks_parser import NXGraphCheckParser
+from checkov.terraform.checks_infra.registry import Registry
 from checkov.terraform.context_parsers.registry import parser_registry
 from checkov.terraform.evaluation.base_variable_evaluation import BaseVariableEvaluation
-from checkov.terraform.parser import Parser
+from checkov.terraform.graph_builder.graph_components.attribute_names import CustomAttributes
+from checkov.terraform.graph_builder.graph_to_tf_definitions import convert_graph_vertices_to_tf_definitions
+from checkov.terraform.graph_builder.local_graph import LocalGraph
+from checkov.terraform.graph_manager import GraphManager
 
 # Allow the evaluation of empty variables
+from checkov.terraform.parser import Parser
 from checkov.terraform.tag_providers import get_resource_tags
 
 dpath.options.ALLOW_EMPTY_STRING_KEYS = True
 
 CHECK_BLOCK_TYPES = frozenset(['resource', 'data', 'provider', 'module'])
-
+graph_registry = Registry(parser=NXGraphCheckParser())
 
 class Runner(BaseRunner):
     check_type = "terraform"
 
-    def __init__(self, parser=Parser()):
+    def __init__(self, parser=Parser(), db_connector=NetworkxConnector(), external_registries=None,
+                 source="Terraform", graph_class=LocalGraph, graph_manager=None):
+        self.external_registries = [] if external_registries is None else external_registries
+        self.graph_class = graph_class
         self.parser = parser
-        self.tf_definitions = {}
+        self.tf_definitions = None
+        self.definitions_context = None
+        self.breadcrumbs = None
         self.definitions_context = {}
         self.evaluations_context: Dict[str, Dict[str, EvaluationContext]] = {}
+        self.graph_manager = graph_manager if graph_manager is not None else GraphManager(source=source,
+                                                                                          db_connector=db_connector)
 
     block_type_registries = {
         'resource': resource_registry,
@@ -43,48 +60,117 @@ class Runner(BaseRunner):
         'module': module_registry,
     }
 
+    def set_external_data(self, tf_definitions: dict, definitions_context: dict, breadcrumbs: dict):
+        self.tf_definitions = tf_definitions
+        self.definitions_context = definitions_context
+        self.breadcrumbs = breadcrumbs
+
     def run(self, root_folder, external_checks_dir=None, files=None, runner_filter=RunnerFilter(), collect_skip_comments=True):
         report = Report(self.check_type)
-        self.tf_definitions = {}
         parsing_errors = {}
-        if external_checks_dir:
-            for directory in external_checks_dir:
-                resource_registry.load_external_checks(directory, runner_filter)
-        if root_folder:
-            root_folder = os.path.abspath(root_folder)
-            
-            self.parser.parse_directory(directory=root_folder,
-                                        out_definitions=self.tf_definitions,
-                                        out_evaluations_context=self.evaluations_context,
-                                        out_parsing_errors=parsing_errors,
-                                        download_external_modules=runner_filter.download_external_modules,
-                                        external_modules_download_path=runner_filter.external_modules_download_path,
-                                        evaluate_variables=runner_filter.evaluate_variables)
-            self.check_tf_definition(report, root_folder, runner_filter, collect_skip_comments)
+        self.load_external_checks(external_checks_dir)
 
-        if files:
-            files = [os.path.abspath(file) for file in files]
-            root_folder = os.path.split(os.path.commonprefix(files))[0]
-            for file in files:
-                if file.endswith(".tf"):
-                    file_parsing_errors = {}
-                    parse_result = self.parser.parse_file(file=file, parsing_errors=file_parsing_errors)
-                    if parse_result is not None:
-                        self.tf_definitions[file] = parse_result
-                    if file_parsing_errors:
-                        parsing_errors.update(file_parsing_errors)
-                        continue
-                    self.check_tf_definition(report, root_folder, runner_filter, collect_skip_comments)
+        if self.definitions_context is None or self.tf_definitions is None or self.breadcrumbs is None:
+            self.tf_definitions = {}
+            logging.info("Scanning root folder and producing fresh tf_definitions and context")
+            if root_folder:
+                root_folder = os.path.abspath(root_folder)
+
+                local_graph, tf_definitions = \
+                    self.graph_manager.build_graph_from_source_directory(root_folder,
+                                                                         local_graph_class=self.graph_class,
+                                                                         parsing_errors=parsing_errors)
+            elif files:
+                files = [os.path.abspath(file) for file in files]
+                root_folder = os.path.split(os.path.commonprefix(files))[0]
+                self.parser.evaluate_variables = False
+                for file in files:
+                    if file.endswith(".tf"):
+                        file_parsing_errors = {}
+                        parse_result = self.parser.parse_file(file=file, parsing_errors=file_parsing_errors)
+                        if parse_result is not None:
+                            self.tf_definitions[file] = parse_result
+                        if file_parsing_errors:
+                            parsing_errors.update(file_parsing_errors)
+                            continue
+                local_graph = self.graph_manager.build_graph_from_tf_definitions(self.tf_definitions)
+            else:
+                raise Exception("Root directory was not specified, files were not specified")
+
+            self.graph_manager.save_graph(local_graph)
+            self.tf_definitions, self.breadcrumbs = convert_graph_vertices_to_tf_definitions(local_graph.vertices, root_folder)
+        else:
+            logging.info(f"Scanning root folder using existing tf_definitions")
+
+        self.check_tf_definition(report, root_folder, runner_filter, collect_skip_comments)
 
         report.add_parsing_errors(parsing_errors.keys())
 
+        graph_report = self.get_graph_checks_report(root_folder, runner_filter)
+        merge_reports(report, graph_report)
+
         return report
 
-    def check_tf_definition(self, report, root_folder, runner_filter, collect_skip_comments=True, external_definitions_context=None):
+    def load_external_checks(self, external_checks_dir: List[str]):
+        if external_checks_dir:
+            for directory in external_checks_dir:
+                resource_registry.load_external_checks(directory)
+                graph_registry.load_external_checks(directory)
+
+    def get_graph_checks_report(self, root_folder, runner_filter: RunnerFilter):
+        report = Report(self.check_type)
+        checks_results = {}
+        for r in self.external_registries + [graph_registry]:
+            r.load_checks()
+            registry_results = r.run_checks(self.graph_manager.get_reader_traversal(), runner_filter)
+            checks_results = {**checks_results, **registry_results}
+
+        for check, check_results in checks_results.items():
+            for check_result in check_results:
+                entity = check_result['entity']
+                entity_context, entity_evaluations = self.get_entity_context_and_evaluations(entity)
+                if entity_context:
+                    full_file_path = entity[CustomAttributes.FILE_PATH]
+                    copy_of_check_result = copy.deepcopy(check_result)
+                    for skipped_check in entity_context.get('skipped_checks', []):
+                        if skipped_check['id'] == check.id:
+                            copy_of_check_result['result'] = CheckResult.SKIPPED
+                            copy_of_check_result['suppress_comment'] = skipped_check['suppress_comment']
+                            break
+                    copy_of_check_result['entity'] = entity.get(CustomAttributes.CONFIG)
+                    record = Record(check_id=check.id,
+                                    check_name=check.name,
+                                    check_result=copy_of_check_result,
+                                    code_block=entity_context.get('code_lines'),
+                                    file_path=f"/{os.path.relpath(full_file_path, root_folder)}",
+                                    file_line_range=[entity_context.get('start_line'),
+                                                     entity_context.get('end_line')],
+                                    resource=".".join(entity_context['definition_path']),
+                                    evaluations=entity_evaluations,
+                                    check_class=check.__class__.__module__,
+                                    file_abs_path=os.path.abspath(full_file_path))
+                    breadcrumb = self.breadcrumbs.get(record.file_path, {}).get(record.resource)
+                    if breadcrumb:
+                        record = GraphRecord(record, breadcrumb)
+
+                    report.add_record(record=record)
+        return report
+
+    def get_entity_context_and_evaluations(self, entity):
+        entity_evaluations = None
+        block_type = entity[CustomAttributes.BLOCK_TYPE]
+        full_file_path = entity[CustomAttributes.FILE_PATH]
+        definition_path = entity[CustomAttributes.BLOCK_NAME].split('.')
+        entity_context = None
+        entity_context_path = [block_type] + definition_path
+        if dpath.search(self.definitions_context.get(full_file_path), entity_context_path):
+            entity_context = dpath.get(self.definitions_context[full_file_path], entity_context_path)
+            entity_context['definition_path'] = definition_path
+        return entity_context, entity_evaluations
+
+    def check_tf_definition(self, report, root_folder, runner_filter, collect_skip_comments=True):
         parser_registry.reset_definitions_context()
-        if external_definitions_context:
-            definitions_context = external_definitions_context
-        else:
+        if not self.definitions_context:
             definitions_context = {}
             for definition in self.tf_definitions.items():
                 definitions_context = parser_registry.enrich_definitions_context(definition, collect_skip_comments)
@@ -95,7 +181,7 @@ class Runner(BaseRunner):
             abs_scanned_file, abs_referrer = self._strip_module_referrer(full_file_path)
             scanned_file = f"/{os.path.relpath(abs_scanned_file, root_folder)}"
             logging.debug(f"Scanning file: {scanned_file}")
-            self.run_all_blocks(definition, definitions_context, full_file_path, root_folder, report,
+            self.run_all_blocks(definition, self.definitions_context, full_file_path, root_folder, report,
                                 scanned_file, runner_filter, abs_referrer)
 
     def run_all_blocks(self, definition, definitions_context, full_file_path, root_folder, report,
@@ -103,11 +189,11 @@ class Runner(BaseRunner):
         if not definition:
             logging.debug("Empty definition, skipping run (root_folder=%s)", root_folder)
             return
-        for block_type in definition.keys():
-            if block_type in CHECK_BLOCK_TYPES:
-                self.run_block(definition[block_type], definitions_context,
-                               full_file_path, root_folder, report,
-                               scanned_file, block_type, runner_filter, None, module_referrer)
+        block_types = set(definition.keys())
+        for block_type in block_types & CHECK_BLOCK_TYPES:
+            self.run_block(definition[block_type], definitions_context,
+                           full_file_path, root_folder, report,
+                           scanned_file, block_type, runner_filter, None, module_referrer)
 
     def run_block(self, entities,
                   definition_context,
@@ -177,7 +263,7 @@ class Runner(BaseRunner):
             results = registry.scan(scanned_file, entity, skipped_checks, runner_filter)
             absolut_scanned_file_path, _ = self._strip_module_referrer(file_path=full_file_path)
             # This duplicates a call at the start of scan, but adding this here seems better than kludging with some tuple return type
-            (entity_type, _, entity_config) = registry.extract_entity_details(entity)
+            (entity_type, entity_name, entity_config) = registry.extract_entity_details(entity)
             tags = get_resource_tags(entity_type, entity_config)
             for check, check_result in results.items():
                 record = Record(check_id=check.id, check_name=check.name, check_result=check_result,
@@ -188,6 +274,9 @@ class Runner(BaseRunner):
                                 entity_tags=tags,
                                 caller_file_path=caller_file_path,
                                 caller_file_line_range=caller_file_line_range)
+                breadcrumb = self.breadcrumbs.get(record.file_path, {}).get('.'.join([entity_type, entity_name]))
+                if breadcrumb:
+                    record = GraphRecord(record, breadcrumb)
                 report.add_record(record=record)
 
     @staticmethod
@@ -216,3 +305,10 @@ class Runner(BaseRunner):
                     if full_file_path in module_content["__resolved__"]:
                         return f"module.{module_name}"
         return None
+
+
+def merge_reports(base_report, report_to_merge):
+    base_report.passed_checks.extend(report_to_merge.passed_checks)
+    base_report.failed_checks.extend(report_to_merge.failed_checks)
+    base_report.skipped_checks.extend(report_to_merge.skipped_checks)
+    base_report.parsing_errors.extend(report_to_merge.parsing_errors)
